/*
ID: espr1t
LANG: C++
TASK: Tetris
KEYWORDS: Checker
*/

#include <ctime>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <algorithm>

#define MAX 131072

#define BOARD_W 10
#define BOARD_H 20

#define MAX_FIG_SIZE 4
#define MAX_BOARD_SIZE 32

char figs[7][MAX_FIG_SIZE + 1][MAX_FIG_SIZE + 1] = {
	 {" #  ",
	  " #  ",
	  " #  ",
	  " #  "},

	 {" ## ",
	  " #  ",
	  " #  ",
	  "    "},

	 {" ## ",
	  "  # ",
	  "  # ",
	  "    "},

	 {" ## ",
	  " ## ",
	  "    ",
	  "    "},

	 {" #  ",
	  " ## ",
	  "  # ",
	  "    "},

	 {" #  ",
	  " ## ",
	  " #  ",
	  "    "},

	 {"  # ",
	  " ## ",
	  " #  ",
	  "    "}
};

int vis[MAX_BOARD_SIZE][MAX_BOARD_SIZE];
int dir[4][2] = { {-1, 0}, {0, 1}, {1, 0}, {0, -1} };

// Color components in different colors
void recurse(char a[MAX_BOARD_SIZE][MAX_BOARD_SIZE], int row, int col, int color)
{
	vis[row][col] = color;
	for (int i = 0; i < 4; i++)
	{
		int nrow = row + dir[i][0]; if (nrow < 0) continue;
		int ncol = col + dir[i][1]; if (ncol < 0) continue;
		if (a[nrow][ncol] == '#' && !vis[nrow][ncol])
			recurse(a, nrow, ncol, color);
	}
}

void dropPieces(char a[MAX_BOARD_SIZE][MAX_BOARD_SIZE])
{
	// While we are erasing rows
	while (1)
	{
		// Find components
		int nextColor = 1;
		memset(vis, 0, sizeof(vis));
		for (int row = 0; row < BOARD_H + 7; row++)
			for (int col = 0; col < BOARD_W; col++)
				if (a[row][col] == '#' && !vis[row][col])
					recurse(a, row, col, nextColor++);

		// Drop down unsupported components
		while (1)
		{
			bool changed = false;
			for (int comp = 1; comp < nextColor; comp++)
			{
				// See if whole component is unsupported (can fall)
				int canFall = 1;
				for (int row = 0; row < BOARD_H + 7; row++)
				{
					for (int col = 0; col < BOARD_W; col++) if (vis[row][col] == comp)
					{
						if (row == 0 || (vis[row - 1][col] != 0 && vis[row - 1][col] != comp))
							{canFall = 0; break;}
					}
					if (!canFall) break;
				}
				if (!canFall) continue;

				// Drop each block of the component 1 square down
				for (int row = 0; row < BOARD_H + 7; row++)
				{
					for (int col = 0; col < BOARD_W; col++) if (vis[row][col] == comp)
					{
						vis[row][col] = 0; a[row][col] = ' ';
						vis[row - 1][col] = comp; a[row - 1][col] = '#';
					}
				}
				changed = true;
			}
			if (!changed) break;
		}

		bool changed = false;
		// Clear all complete rows
		for (int row = 0; row < BOARD_H; row++)
		{
			bool flag = true;
			for (int col = 0; col < BOARD_W; col++)
				if (a[row][col] != '#') {flag = false; break;}
			if (flag)
			{
				changed = true;
				for (int col = 0; col < BOARD_W; col++)
					a[row][col] = ' ';
			}
		}
		if (!changed) break;
	}
}

bool putFigure(char a[MAX_BOARD_SIZE][MAX_BOARD_SIZE], int fig, int rot, int col)
{
	// Copy original figure
	char f[MAX_FIG_SIZE][MAX_FIG_SIZE];
	for (int i = 0; i < MAX_FIG_SIZE; i++)
		for (int c = 0; c < MAX_FIG_SIZE; c++)
			f[i][c] = figs[fig][i][c];

	// Rotate it rot times clockwise
	char t[MAX_FIG_SIZE][MAX_FIG_SIZE];
	for (int r = 0; r < rot; r++)
	{
		for (int row = 0; row < MAX_FIG_SIZE; row++)
			for (int col = 0; col < MAX_FIG_SIZE; col++)
				t[col][MAX_FIG_SIZE - row - 1] = f[row][col];
		memcpy(f, t, sizeof(t));
	}

	int firstRow = MAX_FIG_SIZE, lastRow = -1;
	int firstCol = MAX_FIG_SIZE, lastCol = -1;
	for (int i = 0; i < MAX_FIG_SIZE; i++)
	{
		for (int c = 0; c < MAX_FIG_SIZE; c++) if (f[i][c] == '#')
		{
			firstRow = std::min(firstRow, i); lastRow = std::max(lastRow, i);
			firstCol = std::min(firstCol, c); lastCol = std::max(lastCol, c);
		}
	}

	if (col < 0 || col + lastCol - firstCol >= BOARD_W)
		return false;

	for (int i = lastRow; i >= firstRow; i--)
		for (int c = firstCol; c <= lastCol; c++)
			if (f[i][c] == '#') a[BOARD_H + lastRow - i][col + c - firstCol] = '#';

	dropPieces(a);
	for (int i = 0; i < BOARD_W; i++)
		if (a[BOARD_H][i] != ' ') return false;
	return true;
}

int check(char* inpFile, char* outFile)
{
	FILE* in;
	static char board[MAX_BOARD_SIZE][MAX_BOARD_SIZE];
	memset(board, 0, sizeof(board));
	for (int i = 0; i < MAX_BOARD_SIZE; i++)
		for (int c = 0; c < BOARD_W; c++)
			board[i][c] = ' ';

	// Read input file
	int n;
	static int a[MAX];
	in = fopen(inpFile, "rt");

	if (fscanf(in, "%d", &n) != 1)
	{
		fprintf(stderr, "Could not read number of pieces.\n");
		fprintf(stderr, "  -- wrong input file.\n");
		return -1;
	}
	if (n < 1 || n > 100000)
	{
		fprintf(stderr, "Invalid value for n: %d\n", n);
		fprintf(stderr, "  -- wrong input file.\n");
		return -1;
	}

	for (int i = 0; i < n; i++)
	{
		if (fscanf(in, "%d", &a[i]) != 1)
		{
			fprintf(stderr, "Could not read value for piece %d.\n", i);
			fprintf(stderr, "  -- wrong input file.\n");
			return -1;
		}
		if (a[i] < 1 || a[i] > 7)
		{
			fprintf(stderr, "Invalid value for piece %d: %d.\n", i, a[i]);
			fprintf(stderr, "  -- wrong input file.\n");
			return -1;
		}
	}
	fclose(in);

	// Read output (contestant's) file
	in = fopen(outFile, "rt");

	int ans = 0;
	for (int i = 0; i < n; i++)
	{
		int rot, pos;
		if (fscanf(in, "%d %d", &rot, &pos) != 2)
		{
			//fprintf(stderr, "Could not read values for rotation and position of piece %d.\n", i);
			//fprintf(stderr, "  -- wrong output file.\n");
			//return -1;
			break;
		}
		if (rot < 0 || rot > 3)
		{
			//fprintf(stderr, "Invalid value for rotation of piece %d: %d.\n", i, rot);
			//fprintf(stderr, "  -- wrong output file.\n");
			//return -1;
			break;
		}
		if (pos < 0 || pos > 9)
		{
			//fprintf(stderr, "Invalid value for position of piece %d: %d.\n", i, pos);
			//fprintf(stderr, "  -- wrong output file.\n");
			//return -1;
			break;
		}
		if (!putFigure(board, a[i] - 1, rot, pos))
			break;
		ans++;
	}

	return (int)round(ans * 10.0 / n);
}

int main(int argc, char* argv[])
{
	// Don't know which argument is which file, please fix
	// Write me at espr1t.net@gmail.com if there are issues.

	char* inpFile = argv[1]; // Test input
	char* outFile = argv[2]; // Contestant's solution

	int res = check(inpFile, outFile);
	if (res >= 0) fprintf(stderr, "%d\nAccepted\n", res);
	else fprintf(stderr, "0\nWrong Answer!\n");

	return 0;
}

